/*	Copyright (c) 1984, 1986, 1987, 1988, 1989 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 *		PROPRIETARY NOTICE (Combined)
 *
 *  This source code is unpublished proprietary information
 *  constituting, or derived under license from AT&T's Unix(r) System V.
 *
 *
 *
 *		Copyright Notice
 *
 *  Notice of copyright on this source code product does not indicate
 *  publication.
 *
 *	(c) 1986,1987,1988,1989,1990,1991  Sun Microsystems, Inc.
 *	(c) 1983,1984,1985,1986,1987,1988,1989  AT&T.
 *		All rights reserved.
 */

#ifndef	_NFS_NFS_CLNT_H
#define	_NFS_NFS_CLNT_H

#pragma ident	"@(#)nfs_clnt.h	1.24	95/02/08 SMI"	/* SVr4.0 1.8	*/
/*	nfs_clnt.h 2.28 88/08/19 SMI	*/

#include <vm/page.h>

#ifdef	__cplusplus
extern "C" {
#endif

#define	HOSTNAMESZ	32
#define	ACREGMIN	3	/* min secs to hold cached file attr */
#define	ACREGMAX	60	/* max secs to hold cached file attr */
#define	ACDIRMIN	30	/* min secs to hold cached dir attr */
#define	ACDIRMAX	60	/* max secs to hold cached dir attr */
#define	ACMINMAX	3600	/* 1 hr is longest min timeout */
#define	ACMAXMAX	36000	/* 10 hr is longest max timeout */

#define	NFS_CALLTYPES	3	/* Lookups, Reads, Writes */

/*
 * Fake errno passed back from rfscall to indicate transfer size adjustment
 */
#define	ENFS_TRYAGAIN	999

/*
 * The NFS specific async_reqs structure.
 * Augmented from the async_reqs in vnode.h.
 */

/*
 * Flags to determine whether a request is a read-ahead or a write-behind.
 * They take a slightly different set of args, hence the union.
 */
#define	NFS_READ_AHEAD		0x0
#define	NFS_WRITE_BEHIND	0x1
#define	NFS_PAGEIO		0x2

struct nfs_async_write_req {
	offset_t off;			/* offset in file */
	u_int len;			/* size of i/o request */
	int flags;			/* flags to indicate operation type */
};

struct nfs_async_read_req {
	u_int blkoff;
	struct seg *seg;
	caddr_t addr;
};

struct nfs_pageio_req {
	page_t *pp;			/* page list */
	u_int io_off;			/* offset in file */
	u_int io_len;			/* size of request */
	int flags;
};

struct nfs_async_reqs {
	struct nfs_async_reqs *a_next;	/* pointer to next arg struct */
	struct vnode *a_vp;		/* vnode pointer */
	struct cred *a_cred;		/* cred pointer */
	u_char a_rw;			/* Read-ahead or write-behind ? */
	union {
		struct nfs_async_write_req a_write_args;
		struct nfs_async_read_req a_read_args;
		struct nfs_pageio_req a_pageio_args;
	} a_args;
};
#define	a_nfs_off a_args.a_write_args.off
#define	a_nfs_len a_args.a_write_args.len
#define	a_nfs_flags a_args.a_write_args.flags

#define	a_nfs_blkoff a_args.a_read_args.blkoff
#define	a_nfs_seg a_args.a_read_args.seg
#define	a_nfs_addr a_args.a_read_args.addr

#define	a_nfs_pp	a_args.a_pageio_args.pp
#define	a_nfs_poff	a_args.a_pageio_args.io_off
#define	a_nfs_plen	a_args.a_pageio_args.io_len
#define	a_nfs_pflags	a_args.a_pageio_args.flags

/*
 * NFS private data per mounted file system
 *	The mi_lock mutex protects the following fields:
 *		mi_printed
 *		mi_down
 *		mi_stsize
 *		mi_curread
 *		mi_curwrite
 *		mi_timers
 */
typedef struct mntinfo {
	kmutex_t	 mi_lock;	/* protects mntinfo fields */
	struct knetconfig *mi_knetconfig;	/* bound TLI fd */
	struct netbuf	 mi_addr;	/* server's address */
	struct netbuf	 mi_syncaddr;	/* AUTH_DES time sync addr */
	struct vnode	*mi_rootvp;	/* root vnode */
	u_int		 mi_hard:1;	/* hard or soft mount */
	u_int		 mi_printed:1;	/* not responding message printed */
	u_int		 mi_int:1;	/* interrupts allowed on hard mount */
	u_int		 mi_down:1;	/* server is down */
	u_int		 mi_noac:1;	/* don't cache attributes */
	u_int		 mi_nocto:1;	/* no close-to-open consistency */
	u_int		 mi_dynamic:1;	/* dynamic transfer size adjustment */
	u_int		 mi_llock:1;	/* local locking only (no lockmgr) */
	u_int		 mi_grpid:1;	/* System V group id inheritance */
	u_int		 mi_rpctimesync:1;	/* RPC time sync */
	long		 mi_tsize;	/* transfer size (bytes) */
					/* really read size */
	long		 mi_stsize;	/* server's max transfer size (bytes) */
					/* really write size */
	long		 mi_bsize;	/* server's disk block size */
	int		 mi_mntno;	/* kludge to set client rdev for stat */
	int		 mi_timeo;	/* inital timeout in 10th sec */
	int		 mi_retrans;	/* times to retry request */
	char		 mi_hostname[HOSTNAMESZ];	/* server's hostname */
	char		*mi_netname;	/* server's netname */
	int		 mi_netnamelen;	/* length of netname */
	int		 mi_authflavor;	/* authentication type */
	u_int		 mi_acregmin;	/* min secs to hold cached file attr */
	u_int		 mi_acregmax;	/* max secs to hold cached file attr */
	u_int		 mi_acdirmin;	/* min secs to hold cached dir attr */
	u_int		 mi_acdirmax;	/* max secs to hold cached dir attr */
	/*
	 * Extra fields for congestion control, one per NFS call type,
	 * plus one global one.
	 */
	struct rpc_timers mi_timers[NFS_CALLTYPES+1];
	long		mi_curread;	/* current read size */
	long		mi_curwrite;	/* current write size */
	/*
	 * async I/O management
	 */
	struct nfs_async_reqs *mi_async_reqs; /* head of async request list */
	struct nfs_async_reqs *mi_async_tail; /* tail of async request list */
	kcondvar_t	mi_async_reqs_cv;
	u_short		mi_rthreads;	/* number of active threads */
	u_short		mi_flag;	/* async flags */
	kcondvar_t	mi_flag_cv;
	kmutex_t	mi_async_lock;	/* lock to protect async list */
	struct pathcnf *mi_pathconf;	/* static pathconf kludge */
	clock_t		mi_printftime;	/* last error printf time */
} mntinfo_t;

/*
 * Async flags.
 */
#define	NFS_UNMOUNT_INPROGRESS	0x01

/*
 * vfs pointer to mount info
 */
#define	VFTOMI(vfsp)	((mntinfo_t *)((vfsp)->vfs_data))

/*
 * vnode pointer to mount info
 */
#define	VTOMI(vp)	((mntinfo_t *)(((vp)->v_vfsp)->vfs_data))

/*
 * Mark cached attributes as timed out
 */
#define	PURGE_ATTRCACHE(vp) {					\
	struct rnode *rp;					\
	rp = VTOR(vp);						\
	mutex_enter(&rp->r_statelock);				\
	rp->r_attrtime.tv_sec = hrestime.tv_sec;		\
	rp->r_attrtime.tv_usec = hrestime.tv_nsec / 1000;	\
	mutex_exit(&rp->r_statelock);				\
}

/*
 * Mark cached attributes as uninitialized (must purge all caches first)
 */
#define	INVAL_ATTRCACHE(vp) {					\
	VTOR(vp)->r_attrtime.tv_sec = 0;			\
}

/*
 * If returned error is ESTALE flush all caches.
 */
#ifdef	__STDC__
#define	PURGE_STALE_FH(errno, vp)				\
	if ((errno) == ESTALE) {				\
		extern int nfs_putapage(struct vnode *,		\
			struct page *, u_int *, u_int *, int,	\
			cred_t *);				\
		if (vp->v_type != VCHR)				\
			(void) pvn_vplist_dirty(vp, 0,		\
				nfs_putapage,			\
				B_INVAL| B_TRUNC, CRED());	\
		nfs_purge_caches(vp);				\
}
#else
#define	PURGE_STALE_FH(errno, vp)				\
	if ((errno) == ESTALE) {				\
		extern int nfs_putapage();			\
		if (vp->v_type != VCHR)				\
			(void) pvn_vplist_dirty(vp, 0,		\
				nfs_putapage,			\
				B_INVAL| B_TRUNC, CRED());	\
		nfs_purge_caches(vp);				\
}
#endif	/* __STDC__ */

/*
 * Is cache valid?
 * Swap is always valid, if no attributes (attrtime == 0) or
 * if mtime matches cached mtime it is valid
 * NOTE: mtime is now a timestruc_t.
 * Caller should have the rnode r_protect mutex
 */
#define	CACHE_VALID(rp, mtime, fsize)				\
	((RTOV(rp)->v_flag & VISSWAP) == VISSWAP ||		\
	(rp)->r_attrtime.tv_sec == 0 ||				\
	(((mtime).tv_sec  == (rp)->r_attr.va_mtime.tv_sec &&	\
	(mtime).tv_nsec == (rp)->r_attr.va_mtime.tv_nsec) &&	\
	((fsize) == (rp)->r_attr.va_size)))

#ifdef	__cplusplus
}
#endif

#endif	/* _NFS_NFS_CLNT_H */
