/*
 *	Copyright (c) 1991, Sun Microsystems, Inc.
 */

/*
 * t_lock.h:
 *
 * definitions for thread synchronization primitives:
 * mutexs, semaphores, condition variables, and readers/writer
 * locks.
 */

#ifndef _SYS_T_LOCK_H
#define	_SYS_T_LOCK_H

#pragma ident	"@(#)t_lock.h	1.37	95/05/08 SMI"

#ifndef	_ASM
#include <sys/machlock.h>
#include <sys/dki_lkinfo.h>
#include <sys/sleepq.h>
#include <sys/turnstile.h>
#include <sys/mutex.h>
#endif	/* _ASM */

#ifdef	__cplusplus
extern "C" {
#endif

#ifndef	_ASM

/*
 * Mutual exclusion locks now described in <arch>/sys/mutex.h.
 */

/*
 * Semaphores.
 */
typedef enum {
	SEMA_DEFAULT,
	SEMA_DRIVER
} ksema_type_t;

typedef struct _ksema {
	struct _kthread	*s_slpq;
	int		s_count;
} ksema_t;

typedef enum {
	RW_SLEEP,			/* suspend if lock held */
	RW_SLEEP_STAT,			/* RW_SLEEP with statistics */
	RW_DRIVER_NOSTAT = 2,		/* driver (DDI) version */
	RW_DRIVER_STAT = 3,		/* driver with statistics */
	RW_DEFAULT			/* use statistics if enabled */
} krw_type_t;

typedef enum {
	RW_WRITER,
	RW_READER
} krw_t;

#if defined(_LOCKTEST) || defined(_MPSTATS)
#define	RW_DRIVER	RW_DRIVER_STAT
#else
#define	RW_DRIVER	RW_DRIVER_NOSTAT
#endif


typedef struct _krwlock {
	unsigned char 	type;
	disp_lock_t	rw_wlock;
	turnstile_id_t	waiters;	/* cookie for finding the */
					/* turnstile for this rwlock */
	union {
		struct rwlock_stats *sp;
		struct {
			short	writewanted;	/* num of blocked write reqs */
			short	holdcnt;	/* rlock > 0, wlock == -1 */
		} rw;
	} un;
	/*
	 * The rwlock owner is needed to implement
	 * the limited priority inheritance that we
	 * provide for readers-writers locks.
	 */
	struct _kthread *owner;
} krwlock_t;

/*
 * Statistics package for rwlocks.
 */
struct rwlock_stats {
	krwlock_t	real;			/* real r/w lock */
	lkinfo_t	lkinfo;
	lkstat_t	*lkstat;		/* statistics */
	char		name[LOCK_NAME_LEN+1];	/* name and ending NULL */
	char		flag;			/* flag */
};

/*
 * flags in rwlock_stats.flag
 */
#define	RWSTAT_STARTUP_ALLOC	1	/* allocated from startup pool */

/*
 * Condtion variables.
 */

typedef struct {
	turnstile_id_t	waiters;	/* cookie for finding the turnstile */
					/* for this condition variable. */
} kcondvar_t;

typedef	enum {
	CV_DEFAULT,
	CV_DRIVER
} kcv_type_t;


#if defined(_KERNEL)

extern int ncpus;

/*
 * Tests on lock states.
 */
#define	MUTEX_HELD(x)		(mutex_owned(x))
#define	SEMA_HELD(x)		(sema_held((x)))
#define	RW_READ_HELD(x)		(rw_read_held((x)))
#define	RW_WRITE_HELD(x)	(rw_write_held((x)))
#define	RW_LOCK_HELD(x)		(rw_lock_held((x)))

extern	kmutex_t unsafe_driver;	/* protects MT-unsafe device drivers */

#define	UNSAFE_DRIVER_LOCK_HELD()	(mutex_owned(&unsafe_driver))


/*
 * Dispatcher lock type, macros and routines.
 *
 * disp_lock_t is defined in machlock.h
 */
extern	void	disp_lock_enter(disp_lock_t *);
extern	void	disp_lock_exit(disp_lock_t *);
extern	void	disp_lock_exit_nopreempt(disp_lock_t *);
extern	void	disp_lock_enter_high(disp_lock_t *);
extern	void	disp_lock_exit_high(disp_lock_t *);
extern	void	disp_lock_init(disp_lock_t *lp, char *name);
extern	void	disp_lock_destroy(disp_lock_t *lp);
extern	void	disp_lock_trace(disp_lock_t *lp); /* internal for tracing */

#define	DISP_LOCK_HELD(lp)	LOCK_HELD((lock_t *)(lp))

/*
 * lock function prototypes
 */

extern	void	mutex_init(kmutex_t *, char *, kmutex_type_t, void *);
extern	void	mutex_destroy(kmutex_t *);
extern	void	mutex_enter(kmutex_t *);
extern	int	mutex_tryenter(kmutex_t *);
extern	void	mutex_exit(kmutex_t *);
extern lkstat_t *mutex_stats(kmutex_t *);
extern	int	mutex_owned(kmutex_t *);
extern	struct _kthread *mutex_owner(kmutex_t *);

extern	void	cv_init(kcondvar_t *, char *, kcv_type_t, void *);
extern  void	cv_destroy(kcondvar_t *);
extern	void	cv_wait(kcondvar_t *, kmutex_t *);
extern	int	cv_timedwait(kcondvar_t *, kmutex_t *, long);
extern	int	cv_wait_sig(kcondvar_t *, kmutex_t *);
extern	int	cv_timedwait_sig(kcondvar_t *, kmutex_t *, long *);
extern	int	cv_wait_sig_swap(kcondvar_t *, kmutex_t *);
extern	void	cv_signal(kcondvar_t *);
extern	void	cv_broadcast(kcondvar_t *);
extern  kcondvar_t *cv_alloc(char *, kcv_type_t, void *);
extern  void	cv_free(kcondvar_t *);
extern  struct _kthread *cv_owner(kcondvar_t *);

extern	void	rw_init(krwlock_t *, char *, krw_type_t, void *);
extern	void	rw_destroy(krwlock_t *);
extern	void	rw_enter(krwlock_t *, krw_t);
extern	int	rw_tryenter(krwlock_t *, krw_t);
extern	int	rw_read_locked(krwlock_t *);
extern	void	rw_exit(krwlock_t *);
extern	void	rw_downgrade(krwlock_t *);
extern	int	rw_tryupgrade(krwlock_t *);
extern	int	rw_read_held(krwlock_t *);
extern	int	rw_write_held(krwlock_t *);
extern	int	rw_lock_held(krwlock_t *);
extern	void	rw_mutex_init(void);
extern	struct _kthread *rw_owner(krwlock_t *);

extern	void	sema_init(ksema_t *, unsigned int, char *,
		    ksema_type_t, void *);
extern	void	sema_destroy(ksema_t *);
extern	void	sema_p(ksema_t *);
extern	int	sema_p_sig(ksema_t *);
extern	void	sema_v(ksema_t *);
extern	int	sema_tryp(ksema_t *);
extern	int	sema_held(ksema_t *);
extern  struct _kthread *sema_owner(ksema_t *);
extern	void	sema_mutex_init(void);		/* internal initialization */

#endif	/* defined(_KERNEL) */

#endif	/* _ASM */

#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_T_LOCK_H */
